<?php

namespace Drupal\openwoo_search\Form;

use Drupal\Component\Render\MarkupInterface;
use Drupal\Component\Render\MarkupTrait;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\openwoo_search\OpenWooSearchPluginManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Ajax form load OpenWoo documents.
 */
class OpenWooAjaxForm extends FormBase implements ContainerInjectionInterface {
  use MarkupTrait;

  /**
   * The plugin manager for 'OpenWoo search'.
   *
   * @var \Drupal\openwoo_search\OpenWooSearchPluginManagerInterface
   */
  protected $openWooSearchPluginManager;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Constructs a new OpenWooAjaxForm object.
   *
   * @param \Drupal\openwoo_search\OpenWooSearchPluginManagerInterface $openwoo_search_plugin_manager
   *   The OpenWoo plugin manager.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request_stack.
   */
  public function __construct(OpenWooSearchPluginManagerInterface $openwoo_search_plugin_manager, RequestStack $request_stack) {
    $this->openWooSearchPluginManager = $openwoo_search_plugin_manager;
    $this->requestStack = $request_stack;
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.openwoo_search'),
      $container->get('request_stack'),
    );
  }

  /**
   * {@inheritDoc}
   */
  public function getFormId() {
    return 'openwoo_search_form';
  }

  /**
   * {@inheritDoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form['openwoo_input_wrapper'] = $this->getOpenWooInputWrapper();
    $form['openwoo_wrapper'] = $this->getOpenWooWrapper();

    $form['openwoo_input_wrapper']['search_text'] = $this->getSearchField();
    $form['openwoo_input_wrapper']['category_field'] = $this->getCategoryField();
    $form['openwoo_input_wrapper']['year_field'] = $this->getYearField();

    $form['openwoo_input_wrapper']['submit'] = $this->getSubmitButton();

    $search_results = $this->search();
    if (empty($search_results)) {
      $form['openwoo_wrapper']['openwoo_results'] = $this->getOpenWooResults([], $this->t('Due to technical difficulties we could not retrieve any data from the server. Please try again later or contact technical support if the issue persists.'));
      return $form;
    }

    $form['openwoo_wrapper']['current_page'] = $this->getPageField($search_results['page']);
    $form['openwoo_wrapper']['max_page'] = $this->getPageField($search_results['max_page']);

    $form['openwoo_wrapper']['openwoo_results'] = $this->getOpenWooResults($search_results['results'], $search_results['search_stats']);

    if ($search_results['max_page'] !== 1) {
      $form['openwoo_wrapper']['pager'] = $this->getPagerWrapper();
      $form['openwoo_wrapper']['pager']['previous_page'] = $this->getPreviousButton();
      $form['openwoo_wrapper']['pager']['next_page'] = $this->getNextButton($search_results['page']);
    }

    $form['#cache']['max-age'] = 0;
    return $form;
  }

  /**
   * Get the data for the search field.
   *
   * @return array
   *   The search field.
   */
  protected function getSearchField(): array {
    return [
      '#type' => 'textfield',
      '#size' => 30,
      '#placeholder' => $this->t('Search ...'),
      '#attributes' => [
        'aria-controls' => 'openwoo__results-wrapper',
      ],
    ];
  }

  /**
   * Get the data for the category field.
   *
   * @return array
   *   The category field.
   */
  protected function getCategoryField(): array {
    $active_plugin = $this->openWooSearchPluginManager->getActivePlugin();
    return [
      '#type' => 'select',
      '#options' => $active_plugin->getCategoryOptions(),
      '#empty_value' => NULL,
      '#empty_option' => $this->t('- All categories -'),
      '#attributes' => [
        'aria-controls' => 'openwoo__results-wrapper',
      ],
    ];
  }

  /**
   * Get the data for the year field.
   *
   * @return array
   *   The year field.
   */
  protected function getYearField(): array {
    $active_plugin = $this->openWooSearchPluginManager->getActivePlugin();
    return [
      '#type' => 'select',
      '#options' => $active_plugin->getyearOptions(),
      '#empty_value' => NULL,
      '#empty_option' => $this->t('- All years -'),
      '#attributes' => [
        'aria-controls' => 'openwoo__container',
      ],
    ];
  }

  /**
   * Get the data for the submit button.
   *
   * @return array
   *   The submit button.
   */
  protected function getSubmitButton(): array {
    return [
      '#id' => 'openwoo-submit',
      '#type' => 'submit',
      '#value' => $this->t('Search'),
      '#ajax' => $this->getAjaxCallback($this->t('Loading data...')),
    ];
  }

  /**
   * Get the OpenWoo wrapper.
   *
   * @return array
   *   The OpenWoo wrapper.
   */
  protected function getOpenWooWrapper(): array {
    return [
      '#type' => 'container',
      '#tag' => 'div',
      '#attributes' => [
        'id' => 'openwoo__container',
        'class' => 'openwoo__container',
      ],
    ];
  }

  /**
   * Get the OpenWoo form input wrapper.
   *
   * @return array
   *   The OpenWoo input wrapper.
   */
  protected function getOpenWooInputWrapper(): array {
    return [
      '#type' => 'container',
      '#tag' => 'div',
      '#attributes' => [
        'id' => 'openwoo__form-wrapper',
        'class' => 'openwoo__form-wrapper',
      ],
    ];
  }

  /**
   * Get the pager wrapper.
   *
   * @return array
   *   The pager wrapper.
   */
  protected function getPagerWrapper(): array {
    return [
      '#type' => 'container',
      '#tag' => 'div',
      '#attributes' => [
        'class' => 'openwoo__pager-wrapper',
      ],
    ];
  }

  /**
   * Hidden field for the current page.
   *
   * @param int $page
   *   The current page.
   *
   * @return array
   *   The hidden field for the current page.
   */
  protected function getPageField(int $page): array {
    return [
      '#type' => 'hidden',
      '#default_value' => $page,
    ];
  }

  /**
   * Get the OpenWoo results in a container.
   *
   * @param array $results
   *   The search results.
   * @param \Drupal\Component\Render\MarkupInterface $search_stats
   *   The search statistics.
   *
   * @return array
   *   The container for the results with the results.
   */
  protected function getOpenWooResults(array $results, MarkupInterface $search_stats): array {
    return [
      '#type' => 'container',
      '#tag' => 'div',
      '#theme' => 'openwoo_results',
      '#results' => $results,
      '#search_stats' => $search_stats,
      '#attributes' => [
        'id' => 'openwoo__results-wrapper',
        'aria-live' => 'assertive',
        'aria-atomic' => 'true',
      ],
    ];
  }

  /**
   * Get the previous button.
   *
   * @return array
   *   The data for the previous button.
   */
  protected function getPreviousButton(): array {
    return $this->getPaginationButton('previous_button', $this->t('Previous'), $this->t('Loading previous page...'), [
      'pager-button',
      'pager-previous',
    ], [
      '#states' => [
        'invisible' => [
          ':input[name="current_page"]' => ['value' => '1'],
        ],
      ],
    ]);
  }

  /**
   * Get the next button.
   *
   * @param int $page
   *   The current pages.
   *
   * @return array
   *   The data for the next button.
   */
  protected function getNextButton(int $page): array {
    return $this->getPaginationButton('next_button', $this->t('Next'), $this->t('Loading next page...'), [
      'pager-button',
      'pager-next',
    ], [
      '#states' => [
        'invisible' => [
          ':input[name="max_page"]' => ['value' => $page],
        ],
      ],
    ]);
  }

  /**
   * Get the pagination button.
   *
   * @param string $id
   *   The id for the button.
   * @param \Drupal\Component\Render\MarkupInterface $value
   *   The value of the button.
   * @param \Drupal\Component\Render\MarkupInterface $message
   *   The loading message.
   * @param array $class
   *   The classes.
   * @param array $field_options
   *   Extra options for the field.
   *
   * @return array
   *   The pagination button.
   */
  protected function getPaginationButton(string $id, MarkupInterface $value, MarkupInterface $message, array $class, array $field_options = []): array {
    return $field_options + [
      '#type' => 'button',
      '#id' => $id,
      '#value' => $value,
      '#ajax' => $this->getAjaxCallback($message, 'fade'),
      '#attributes' => ['class' => $class],
    ];
  }

  /**
   * Get the ajax callback settings.
   *
   * @param \Drupal\Component\Render\MarkupInterface|null $message
   *   The message.
   * @param string|null $effect
   *   The effect.
   *
   * @return array
   *   The ajax callback array.
   */
  protected function getAjaxCallback(MarkupInterface $message = NULL, string $effect = NULL): array {
    $callback = [
      'callback' => [$this, 'openwooAjaxCallback'],
      'wrapper' => 'openwoo__container',
      'progress' => [
        'type' => 'throbber',
        'message' => $message,
      ],
      'effect' => $effect,
      'event' => 'click',
    ];

    if (!$effect) {
      unset($callback['effect']);
    }

    return $callback;
  }

  /**
   * Ajax callback to fetch the woo results.
   *
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @return array
   *   The elements to render in the ajax wrapper.
   */
  public function openwooAjaxCallback(array $form, FormStateInterface &$form_state): array {
    $search_text = $form_state->getValue('search_text');
    $selected_year = $form_state->getValue('year_field');
    $selected_category = $form_state->getValue('category_field');
    $page = $form_state->getValue('current_page');

    $triggering_element = $form_state->getTriggeringElement();
    if ($triggering_element['#id'] === "previous_button") {
      $page--;
      $page = max($page, 1);
    }
    elseif ($triggering_element['#id'] === "next_button") {
      $page++;
    }
    else {
      // When submit button or facet is used, reset to page 1.
      $page = 1;
    }

    $active_plugin = $this->openWooSearchPluginManager->getActivePlugin();

    $json = $active_plugin->search($search_text, $selected_year, $selected_category, $page);
    $search_results = $active_plugin->parseResults($json);
    $pager = $active_plugin->getPager($json);

    $form['openwoo_wrapper']['openwoo_results']['#results'] = $search_results;
    $form['openwoo_wrapper']['openwoo_results']['#search_stats'] = $active_plugin->getSearchStats($json, $search_text);
    $form['openwoo_wrapper']['current_page']['#value'] = $page;
    $form['openwoo_wrapper']['pager']['next_page']['#access'] = $page < $pager['number_of_pages'];
    $form['openwoo_wrapper']['pager']['previous_page']['#access'] = $page > 1;
    $form_state->setRebuild();
    return $form['openwoo_wrapper'];
  }

  /**
   * {@inheritDoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

  }

  /**
   * Get the search results, pager and facets.
   *
   * @return array
   *   The search array.
   */
  protected function search(): array {
    $request = $this->requestStack->getCurrentRequest();
    $search_text = $request->get('search_text') ?? '';
    $selected_year = $request->get('year_field') ?? NULL;
    $selected_category = $request->get('category_field') ?? NULL;

    $active_plugin = $this->openWooSearchPluginManager->getActivePlugin();

    $json = $active_plugin->search($search_text, $selected_year, $selected_category);
    if ($json === NULL) {
      return [];
    }
    $pager = $active_plugin->getPager($json);
    $search_stats = $active_plugin->getSearchStats($json, $search_text);

    return [
      'result_count' => $pager['items'],
      'result_limit' => $pager['results_per_page'],
      'page' => $pager['current_page'],
      'max_page' => $pager['number_of_pages'],
      'results' => $active_plugin->parseResults($json),
      'search_stats' => $search_stats,
    ];
  }

}
